WITH   Ada.Unchecked_Conversion ;
WITH   Interfaces.C ;
WITH   Stdarg ;
WITH   System ;
WITH   Win32 ;
WITH   Win32.Crt.Strings ;
WITH   Win32.Windef ;
WITH   Win32.Wingdi ;
WITH   Win32.Winuser ;

PACKAGE BODY   PrimeNb_Pkg  IS

   FUNCTION   CP   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CP;

   FUNCTION   CPS   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CPS;

   USE TYPE   Win32.CHAR_Array ;      --- To get "&"

   Nul : Win32.Char RENAMES Win32.Nul ;

   wm_MyAction : CONSTANT := Win32.Winuser.WM_USER + 101 ;

   C_MainMyTitle : CONSTANT Win32.CHAR_Array := "***** PRIME NUMBERS: "  &  Nul ;
   C_MainNbFmt   : CONSTANT Win32.CHAR_Array := "%ld"  &  Nul ;
   C_SpaceZ : CONSTANT Win32.CHAR_Array := " "  &  Nul ;

   L_PaintStruct : ALIASED Win32.Winuser.PAINTSTRUCT ;
   L_Rect        : ALIASED Win32.Windef.RECT := (0,0,0,0);

   L_Buf25  : ALIASED Win32.CHAR_Array(0..25);

   IsPrimeArray : Array(1..1000) of Boolean ;
   j , jMax : Integer ;

   FUNCTION   MainWindow_Proc   ( I_HWnd    : Win32.Windef.HWND
                                ; I_Message : Win32.UINT
                                ; I_WParam  : Win32.WPARAM
                                ; I_LParam  : Win32.LPARAM )  RETURN  Win32.LRESULT  IS
      FUNCTION   "&"  IS  new Stdarg.Concat ( Win32.LONG );

      L_HDC : Win32.Windef.HDC ;
   BEGIN
      CASE I_Message IS
         WHEN wm_MyAction =>
            IgnoredP := Win32.Crt.Strings.StrCpy ( CPS(G_MainMsgBuf) , CP(C_MainMyTitle) );

            ----- LOOP 1 , On each number
            FOR i in 1..1000 LOOP

               ----- Could be a prime number
               IsPrimeArray(i) := TRUE ;

               j    := 2 ;
               jMax := Integer( i / 2 );

               ----- LOOP 2 , Search a divisor
               WHILE IsPrimeArray(i) and (j <= jMax) LOOP
                  ----- "j" can divide "i" ?
                  IF (i mod j) = 0 THEN
                     IsPrimeArray(i) := False ;
                  END IF;
                  j := j + 1 ;
               END LOOP;

               ----- Display only prime numbers
               IF IsPrimeArray(i) = TRUE THEN
                  IgnoredI := Win32.Winuser.wvsprintf ( CPS(L_Buf25) , CP(C_MainNbFmt)
                                                      , Stdarg.Empty & Win32.LONG(i) );
                  IgnoredP := Win32.Crt.Strings.StrCat ( CPS(G_MainMsgBuf) , CP(L_Buf25) );
                  IgnoredP := Win32.Crt.Strings.StrCat ( CPS(G_MainMsgBuf) , CP(C_SpaceZ) );
               END IF;
            END LOOP;

            ----- Force paint again
            IgnoredB := Win32.Winuser.GetClientRect ( I_HWnd , L_Rect'Access );
            IgnoredB := Win32.Winuser.InvalidateRect ( I_HWnd , L_Rect'Access , Win32.FALSE );

         WHEN Win32.Winuser.WM_PAINT =>
            L_HDC := Win32.Winuser.BeginPaint ( I_HWnd , L_PaintStruct'Access );
            IgnoredB := Win32.Winuser.GetClientRect ( I_HWnd , L_Rect'Access );

            IgnoredI := Win32.Winuser.DrawText ( L_HDC
                                               , CP(G_MainMsgBuf)
                                               , Interfaces.C.Int( Win32.Crt.Strings.StrLen(CP(G_MainMsgBuf)) )
                                               , L_Rect'Access
                                               , Win32.Winuser.DT_WORDBREAK );

            IgnoredB := Win32.Winuser.EndPaint ( L_HDC , L_PaintStruct'Access );

            ----- Only once...
            IF not G_MainPaintedOnceQ THEN
               G_MainPaintedOnceQ := Standard.TRUE ;
               IgnoredB := Win32.Winuser.PostMessage ( I_HWnd , wm_MyAction , 0 , 0 );      --- "Indirect" call...
            END IF;

         WHEN Win32.Winuser.WM_SIZE =>      --- When window is resized,...
            ----- Force paint again
            IgnoredB := Win32.Winuser.GetClientRect ( I_HWnd , L_Rect'Access );
            IgnoredB := Win32.Winuser.InvalidateRect ( I_HWnd , L_Rect'Access , Win32.FALSE );

         WHEN Win32.Winuser.WM_DESTROY =>
            Win32.Winuser.PostQuitMessage(0);

         WHEN OTHERS =>
            RETURN  Win32.Winuser.DefWIndowProc ( I_HWnd , I_Message , I_WParam , I_LParam );
      END CASE;

      ----- Default result : Msg processed!!!
      RETURN  0 ;
   END MainWindow_Proc;
END PrimeNb_Pkg;
